@@ -1,24 +1,29 @@
-/***************************************************************************
-    copyright            : (C) 2005 by Brian Nickel
-    email                : brian.nickel@gmail.com
-    based on             : oggfile.cpp from TagLib
- ***************************************************************************/
-
-/***************************************************************************
- *   This library is free software; you can redistribute it and/or modify  *
- *   it  under the terms of the GNU Lesser General Public License version  *
- *   2.1 as published by the Free Software Foundation.                     *
- *                                                                         *
- *   This library is distributed in the hope that it will be useful, but   *
- *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
- *   Lesser General Public License for more details.                       *
- *                                                                         *
- *   You should have received a copy of the GNU Lesser General Public      *
- *   License along with this library; if not, write to the Free Software   *
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
- *   USA                                                                   *
- ***************************************************************************/
+//
+// File.cs:
+//
+// Author:
+//   Brian Nickel (brian.nickel@gmail.com)
+//
+// Original Source:
+//   oggfile.cpp from TagLib
+//
+// Copyright (C) 2005-2007 Brian Nickel
+// Copyright (C) 2003 Scott Wheeler (Original Implementation)
+//
+// This library is free software; you can redistribute it and/or modify
+// it  under the terms of the GNU Lesser General Public License version
+// 2.1 as published by the Free Software Foundation.
+//
+// This library is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+// Lesser General Public License for more details.
+//
+// You should have received a copy of the GNU Lesser General Public
+// License along with this library; if not, write to the Free Software
+// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+// USA
+//
 
 using System.Collections;
 using System.Collections.Generic;
@@ -48,14 +53,18 @@ namespace TagLib.Ogg
       public File (string path) : this (path, ReadStyle.Average)
       {}
       
-      public File (File.IFileAbstraction abstraction, ReadStyle propertiesStyle) : base (abstraction)
-      {
-         Mode = AccessMode.Read;
-         tag = new GroupedComment ();
-         Read (propertiesStyle);
-         Mode = AccessMode.Closed;
-         TagTypesOnDisk = TagTypes;
-      }
+		public File (File.IFileAbstraction abstraction,
+		             ReadStyle propertiesStyle) : base (abstraction)
+		{
+			Mode = AccessMode.Read;
+			try {
+				tag = new GroupedComment ();
+				Read (propertiesStyle);
+				TagTypesOnDisk = TagTypes;
+			} finally {
+				Mode = AccessMode.Closed;
+			}
+		}
       
       public File (File.IFileAbstraction abstraction) : this (abstraction, ReadStyle.Average)
       {}
@@ -100,56 +109,68 @@ namespace TagLib.Ogg
             tag.Clear ();
       }
       
-      public override void Save ()
-      {
-         Mode = AccessMode.Write;
-         
-         long end;
-         List<Page> pages = new List<Page> ();
-         Dictionary<uint, Bitstream> streams = ReadStreams (pages, out end);
-         Dictionary<uint, Paginator> paginators = new Dictionary<uint, Paginator> ();
-         List<List<Page>> new_pages = new List<List<Page>> ();
-         
-         foreach (Page page in pages)
-         {
-            uint id = page.Header.StreamSerialNumber;
-            if (!paginators.ContainsKey (id))
-               paginators.Add (id, new Paginator (streams [id].Codec));
-            
-            paginators [id].AddPage (page);
-         }
-         
-         foreach (uint id in paginators.Keys)
-         {
-            paginators [id].SetComment (tag.GetComment (id));
-            new_pages.Add (new List<Page> (paginators [id].Paginate ()));
-         }
-         
-         ByteVector output = new ByteVector ();
-         bool empty;
-         do
-         {
-            empty = true;
-            foreach (List<Page> stream_pages in new_pages)
-            {
-               if (stream_pages.Count == 0)
-                  continue;
-               
-               output.Add (stream_pages [0].Render ());
-               stream_pages.RemoveAt (0);
-               
-               if (stream_pages.Count != 0)
-                  empty = false;
-            }
-         } while (!empty);
-         
-         Insert (output, 0, end);
-         InvariantStartPosition = output.Count;
-         InvariantEndPosition = Length;
-         
-         Mode = AccessMode.Closed;
-         TagTypesOnDisk = TagTypes;
-      }
+		public override void Save ()
+		{
+			Mode = AccessMode.Write;
+			try {
+				long end;
+				List<Page> pages = new List<Page> ();
+				Dictionary<uint, Bitstream> streams =
+					ReadStreams (pages, out end);
+				Dictionary<uint, Paginator> paginators =
+					new Dictionary<uint, Paginator> ();
+				List<List<Page>> new_pages =
+					new List<List<Page>> ();
+				Dictionary<uint, int> shifts =
+					new Dictionary<uint, int> ();
+				
+				foreach (Page page in pages) {
+					uint id = page.Header.StreamSerialNumber;
+					if (!paginators.ContainsKey (id))
+						paginators.Add (id, new Paginator (
+							streams [id].Codec));
+					
+					paginators [id].AddPage (page);
+				}
+
+				foreach (uint id in paginators.Keys) {
+					paginators [id].SetComment (
+						tag.GetComment (id));
+					int shift;
+					new_pages.Add (new List<Page> (
+						paginators [id].Paginate (out shift)));
+					shifts.Add (id, shift);
+				}
+
+				ByteVector output = new ByteVector ();
+				bool empty;
+				do {
+					empty = true;
+					foreach (List<Page> stream_pages in new_pages) {
+						if (stream_pages.Count == 0)
+							continue;
+
+						output.Add (stream_pages [0]
+							.Render ());
+						stream_pages.RemoveAt (0);
+
+						if (stream_pages.Count != 0)
+							empty = false;
+					}
+				} while (!empty);
+
+				Insert (output, 0, end);
+				InvariantStartPosition = output.Count;
+				InvariantEndPosition = Length;
+
+				TagTypesOnDisk = TagTypes;
+				
+				Page.OverwriteSequenceNumbers (this,
+					output.Count, shifts);
+			} finally {
+				Mode = AccessMode.Closed;
+			}
+		}
       
       private Dictionary<uint, Bitstream> ReadStreams (List<Page> pages, out long end)
       {
