@@ -60,7 +60,7 @@ struct GstCdRipper {
     gchar *encoder_pipeline;
     
     GstElement *pipeline;
-    GstElement *cdparanoia;
+    GstElement *cddasrc;
     GstElement *encoder;
     GstElement *filesink;
     
@@ -111,7 +111,7 @@ gst_cd_ripper_iterate_timeout(GstCdRipper *ripper)
         return TRUE;
     }
 
-    if(!gst_element_query_position(ripper->cdparanoia, &format, &position)) {
+    if(!gst_element_query_position(ripper->cddasrc, &format, &position)) {
         return TRUE;
     }
     
@@ -191,55 +191,29 @@ gst_cd_ripper_bus_callback(GstBus *bus, GstMessage *message, gpointer data)
 }
 
 static GstElement *
-gst_cd_ripper_build_encoder(const gchar *encoder_pipeline)
+gst_cd_ripper_build_encoder(const gchar *pipeline, GError **error_out)
 {
-    GstElement *encoder = NULL;
-    gchar *pipeline = NULL;
+    GstElement *encoder;
     GError *error = NULL;
-    
-    // FIXME: ugly
-    if(strstr(encoder_pipeline, "vorbisenc") != NULL 
-        && strstr(encoder_pipeline, "oggmux") == NULL) {
-        g_warning("vorbisenc added without oggmux, attempting to insert oggmux element in pipeline");
-        pipeline = g_strdup_printf("audioconvert ! %s ! oggmux", encoder_pipeline);
-    } else if((strstr(encoder_pipeline, "lame") != NULL || strstr(encoder_pipeline, "xingenc") != NULL) &&
-        strstr(encoder_pipeline, "mux") == NULL) {
-        const gchar *muxer_names [] = { "id3v2mux", "taglibid3mux", "id3mux", NULL};
-        gint i;
-        
-        for(i = 0; muxer_names[i] != NULL; i++) {
-            GstElementFactory *id3mux = gst_element_factory_find(muxer_names[i]);
-            if(id3mux == NULL) {
-                continue;
-            }
-            
-            g_warning("MP3 encoder added without %s, attempting to insert %s element in pipeline",
-                muxer_names[i], muxer_names[i]);
-            pipeline = g_strdup_printf("audioconvert ! %s ! %s", encoder_pipeline, muxer_names[i]);
-            gst_object_unref(GST_OBJECT(id3mux));
-            break;
-        }
-    }
-    
-    if(pipeline == NULL) {
-        pipeline = g_strdup_printf("audioconvert ! %s", encoder_pipeline);
-    }
-    
+   
     encoder = gst_parse_bin_from_description(pipeline, TRUE, &error);
-    g_free(pipeline);
     
     if(error != NULL) {
+        if(error_out != NULL) {
+            *error_out = error;
+        }
         return NULL;
     }
     
     return encoder;
-}    
+}
 
 static gboolean
 gst_cd_ripper_build_pipeline(GstCdRipper *ripper)
 {
     GstElement *queue;
     GstElement *mbtrm;
+    GError *error = NULL;
     
     g_return_val_if_fail(ripper != NULL, FALSE);
         
@@ -249,14 +223,17 @@ gst_cd_ripper_build_pipeline(GstCdRipper *ripper)
         return FALSE;
     }
 
-    ripper->cdparanoia = gst_element_factory_make("cdparanoiasrc", "cdparanoia");
-    if(ripper->cdparanoia == NULL) {
-        gst_cd_ripper_raise_error(ripper, _("Could not initialize cdparanoia"), NULL);
+    ripper->cddasrc = gst_element_make_from_uri(GST_URI_SRC, "cdda://1", "cddasrc");
+    if(ripper->cddasrc == NULL) {
+        gst_cd_ripper_raise_error(ripper, _("Could not initialize element from cdda URI"), NULL);
         return FALSE;
     }
   
-    g_object_set(G_OBJECT(ripper->cdparanoia), "device", ripper->device, NULL);
-    g_object_set(G_OBJECT(ripper->cdparanoia), "paranoia-mode", ripper->paranoia_mode, NULL);
+    g_object_set(G_OBJECT(ripper->cddasrc), "device", ripper->device, NULL);
+    
+    if(g_object_class_find_property(G_OBJECT_GET_CLASS(ripper->cddasrc), "paranoia-mode")) {
+        g_object_set(G_OBJECT(ripper->cddasrc), "paranoia-mode", ripper->paranoia_mode, NULL);
+    }
     
     ripper->track_format = gst_format_get_by_nick("track");
     
@@ -272,9 +249,9 @@ gst_cd_ripper_build_pipeline(GstCdRipper *ripper)
     g_signal_connect(G_OBJECT(mbtrm), "have-trm-id",
         G_CALLBACK(gst_cd_ripper_have_trm_id_cb), ripper);
     
-    ripper->encoder = gst_cd_ripper_build_encoder(ripper->encoder_pipeline);
+    ripper->encoder = gst_cd_ripper_build_encoder(ripper->encoder_pipeline, &error);
     if(ripper->encoder == NULL) {
-        gst_cd_ripper_raise_error(ripper, _("Could not create encoder pipeline"), NULL);
+        gst_cd_ripper_raise_error(ripper, _("Could not create encoder pipeline"), error->message);
         return FALSE;
     }
     
@@ -296,18 +273,33 @@ gst_cd_ripper_build_pipeline(GstCdRipper *ripper)
         G_CALLBACK(gst_cd_ripper_gvfs_allow_overwrite_cb), ripper);
     
     gst_bin_add_many(GST_BIN(ripper->pipeline),
-        ripper->cdparanoia,
+        ripper->cddasrc,
         mbtrm,
         queue,
         ripper->encoder,
         ripper->filesink,
         NULL);
         
-    if(!gst_element_link_many(ripper->cdparanoia, mbtrm, queue, ripper->encoder, ripper->filesink, NULL)) {
-        gst_cd_ripper_raise_error(ripper, _("Could not link pipeline elements"), NULL);
+    if(!gst_element_link(ripper->cddasrc, mbtrm)) {
+        gst_cd_ripper_raise_error(ripper, _("Could not link cddasrcsrc to mbtrm"), NULL);
         return FALSE;
     }
-
+        
+    if(!gst_element_link(mbtrm, queue)) {
+        gst_cd_ripper_raise_error(ripper, _("Could not link mbtrm to queue"), NULL);
+        return FALSE;
+    }
+        
+    if(!gst_element_link(queue, ripper->encoder)) {
+        gst_cd_ripper_raise_error(ripper, _("Could not link queue to encoder"), NULL);
+        return FALSE;
+    }
+        
+    if(!gst_element_link(ripper->encoder, ripper->filesink)) {
+        gst_cd_ripper_raise_error(ripper, _("Could not link encoder to gnomevfssink"), NULL);
+        return FALSE;
+    }
+    
     gst_bus_add_watch(gst_pipeline_get_bus(GST_PIPELINE(ripper->pipeline)), 
         gst_cd_ripper_bus_callback, ripper);
 
@@ -332,7 +324,7 @@ gst_cd_ripper_new(gchar *device, gint paranoia_mode, gchar *encoder_pipeline)
     ripper->encoder_pipeline = g_strdup(encoder_pipeline);
     
     ripper->pipeline = NULL;
-    ripper->cdparanoia = NULL;
+    ripper->cddasrc = NULL;
     ripper->encoder = NULL;
     ripper->filesink = NULL;
     
@@ -345,20 +337,36 @@ gst_cd_ripper_new(gchar *device, gint paranoia_mode, gchar *encoder_pipeline)
     return ripper;
 }
 
-void
-gst_cd_ripper_free(GstCdRipper *ripper)
+void 
+gst_cd_ripper_cancel(GstCdRipper *ripper)
 {
     g_return_if_fail(ripper != NULL);
     
     gst_cd_ripper_stop_iterate_timeout(ripper);
     
-    if(GST_IS_ELEMENT(ripper->pipeline)) {
+    if(ripper->pipeline != NULL && GST_IS_ELEMENT(ripper->pipeline)) {
         gst_element_set_state(GST_ELEMENT(ripper->pipeline), GST_STATE_NULL);
         gst_object_unref(GST_OBJECT(ripper->pipeline));
+        ripper->pipeline = NULL;
     }
     
-    g_free(ripper->device);
-    g_free(ripper->encoder_pipeline);
+    g_remove(ripper->output_uri);
+}
+
+void
+gst_cd_ripper_free(GstCdRipper *ripper)
+{
+    g_return_if_fail(ripper != NULL);
+    
+    gst_cd_ripper_cancel(ripper);
+    
+    if(ripper->device != NULL) {
+        g_free(ripper->device);
+    }
+    
+    if(ripper->encoder_pipeline != NULL) {
+        g_free(ripper->encoder_pipeline);
+    }
     
     g_free(ripper);
 }
@@ -366,12 +374,13 @@ gst_cd_ripper_free(GstCdRipper *ripper)
 gboolean
 gst_cd_ripper_rip_track(GstCdRipper *ripper, gchar *uri, gint track_number, 
     gchar *md_artist, gchar *md_album, gchar *md_title, gchar *md_genre,
-    gint md_track_number, gint md_track_count, gpointer user_info)
+    gint md_track_number, gint md_track_count, gint md_year, gpointer user_info)
 {
     GstIterator *bin_iterator;
     GstElement *bin_element;
     gboolean can_tag = FALSE;
     gboolean iterate_done = FALSE;
+    GDate *date;
 
     g_return_val_if_fail(ripper != NULL, FALSE);
 
@@ -382,6 +391,8 @@ gst_cd_ripper_rip_track(GstCdRipper *ripper, gchar *uri, gint track_number,
     // initialize the pipeline, set the sink output location
     gst_element_set_state(ripper->filesink, GST_STATE_NULL);
     g_object_set(G_OBJECT(ripper->filesink), "location", uri, NULL);
+
+    date = g_date_new();
     
     // find an element to do the tagging and set tag data
     bin_iterator = gst_bin_iterate_all_by_interface(GST_BIN(ripper->encoder), GST_TYPE_TAG_SETTER);
@@ -398,10 +409,19 @@ gst_cd_ripper_rip_track(GstCdRipper *ripper, gchar *uri, gint track_number,
                     GST_TAG_ENCODER, _("Banshee"),
                     GST_TAG_ENCODER_VERSION, VERSION,
                     NULL);
-                    
-                if(md_genre && strlen(md_genre) == 0) {
+
+                if (g_date_valid_dmy(1, 1, md_year)) {
+                    g_date_clear(date, 1);
+                    g_date_set_dmy(date, 1, 1, md_year);
                     gst_tag_setter_add_tags(GST_TAG_SETTER(bin_element),
-                        GST_TAG_MERGE_APPEND,
+                        GST_TAG_MERGE_REPLACE_ALL,
+                        GST_TAG_DATE, date,
+                        NULL);
+                }
+                    
+                if(md_genre && strlen(md_genre) > 0) {
+                    gst_tag_setter_add_tags(GST_TAG_SETTER(bin_element),
+                        GST_TAG_MERGE_REPLACE_ALL,
                         GST_TAG_GENRE, md_genre,
                         NULL);
                 }
@@ -417,6 +437,8 @@ gst_cd_ripper_rip_track(GstCdRipper *ripper, gchar *uri, gint track_number,
                 break;
         }
     }
+
+    g_date_free (date);
     
     gst_iterator_free(bin_iterator);
     
@@ -425,7 +447,7 @@ gst_cd_ripper_rip_track(GstCdRipper *ripper, gchar *uri, gint track_number,
     }
     
     // start the ripping
-    g_object_set(G_OBJECT(ripper->cdparanoia), "track", track_number, NULL);
+    g_object_set(G_OBJECT(ripper->cddasrc), "track", track_number, NULL);
     
     gst_element_set_state(ripper->pipeline, GST_STATE_PLAYING);
     gst_cd_ripper_start_iterate_timeout(ripper);
@@ -433,20 +455,6 @@ gst_cd_ripper_rip_track(GstCdRipper *ripper, gchar *uri, gint track_number,
     return TRUE;
 }
 
-void 
-gst_cd_ripper_cancel(GstCdRipper *ripper)
-{
-    g_return_if_fail(ripper != NULL);
-    gst_cd_ripper_stop_iterate_timeout(ripper);
-    
-    if(GST_IS_ELEMENT(ripper->pipeline)) {
-        gst_element_set_state(GST_ELEMENT(ripper->pipeline), GST_STATE_NULL);
-        gst_object_unref(GST_OBJECT(ripper->pipeline));
-    }
-    
-    g_remove(ripper->output_uri);
-}
-
 void
 gst_cd_ripper_set_progress_callback(GstCdRipper *ripper, 
     GstCdRipperProgressCallback cb)
