@@ -38,6 +38,10 @@
 
 #include <gst/gst.h>
 
+#ifdef HAVE_GST_PBUTILS
+#  include <gst/pbutils/pbutils.h>
+#endif
+
 #include "gst-mbtrm.h"
 
 static gboolean gstreamer_initialized = FALSE;
@@ -70,27 +74,14 @@ void gstreamer_initialize()
     }
 
     gst_init(NULL, NULL);
+    
+    #ifdef HAVE_GST_PBUTILS
+    gst_pb_utils_init();
+    #endif
+    
     _gst_plugin_register_static(&gst_mbtrm_plugin_desc);
-    
-    gstreamer_initialized = TRUE;
-}
 
-gboolean
-gstreamer_test_encoder(gchar *encoder_pipeline)
-{
-    GstElement *element = NULL;
-    gchar *pipeline;
-    GError *error = NULL;
-    
-    pipeline = g_strdup_printf("audioconvert ! %s", encoder_pipeline);
-    element = gst_parse_launch(pipeline, &error);
-    g_free(pipeline);
-    
-    if(element != NULL) {
-        gst_object_unref(GST_OBJECT(element));
-    }
-    
-    return error == NULL;
+    gstreamer_initialized = TRUE;
 }
 
 gboolean 
@@ -100,10 +91,11 @@ gstreamer_test_pipeline(gchar *pipeline)
     GError *error = NULL;
     
     element = gst_parse_launch(pipeline, &error);
-    
+
     if(element != NULL) {
         gst_object_unref(GST_OBJECT(element));
     }
     
     return error == NULL;
 }
+
