@@ -26,6 +26,10 @@
  *  DEALINGS IN THE SOFTWARE.
  */
  
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -34,6 +38,15 @@
 
 #include <gst/gst.h>
 
+#ifdef HAVE_GST_PBUTILS
+#  include <gst/pbutils/pbutils.h>
+#endif
+
+#include <gdk/gdk.h>
+#ifdef GDK_WINDOWING_X11
+#include <gdk/gdkx.h>
+#endif
+
 #include "gst-tagger.h"
 
 #define IS_GST_PLAYBACK(e) (e != NULL)
@@ -43,7 +56,7 @@ typedef struct GstPlayback GstPlayback;
 
 typedef void (* GstPlaybackEosCallback) (GstPlayback *engine);
 typedef void (* GstPlaybackErrorCallback) (GstPlayback *engine, 
-    const gchar * error, const gchar *debug);
+    GQuark domain, gint code, const gchar *error, const gchar *debug);
 typedef void (* GstPlaybackStateChangedCallback) (
     GstPlayback *engine, GstState old_state, 
     GstState new_state, GstState pending_state);
@@ -58,16 +71,39 @@ struct GstPlayback {
     guint iterate_timeout_id;
     gchar *cdda_device;
     
+    GstState target_state;
+    gboolean buffering;
+    
     GstPlaybackEosCallback eos_cb;
     GstPlaybackErrorCallback error_cb;
     GstPlaybackStateChangedCallback state_changed_cb;
     GstPlaybackIterateCallback iterate_cb;
     GstPlaybackBufferingCallback buffering_cb;
     GstTaggerTagFoundCallback tag_found_cb;
+    
+    GdkWindow *window;
+    GSList *missing_element_details;
+    gboolean install_plugins_noprompt;
+    
+    #ifdef HAVE_GST_PBUTILS
+    GstInstallPluginsContext *install_plugins_context;
+    #endif
 };
 
 // private methods
 
+static void
+gst_playback_nuke_slist(GSList *list)
+{   
+    GSList *node = list;
+    
+    for(; node != NULL; node = node->next) {
+        g_free(node->data);
+    }
+    
+    g_slist_free(list);
+}
+
 static void
 gst_playback_destroy_pipeline(GstPlayback *engine)
 {
@@ -78,6 +114,7 @@ gst_playback_destroy_pipeline(GstPlayback *engine)
     }
     
     if(GST_IS_ELEMENT(engine->playbin)) {
+        engine->target_state = GST_STATE_NULL;
         gst_element_set_state(engine->playbin, GST_STATE_NULL);
         gst_object_unref(GST_OBJECT(engine->playbin));
     }
@@ -85,6 +122,89 @@ gst_playback_destroy_pipeline(GstPlayback *engine)
     engine->playbin = NULL;
 }
 
+#ifdef HAVE_GST_PBUTILS
+static gchar **
+gst_playback_missing_element_details_vectorize(const GSList *elements)
+{
+    GPtrArray *vector = g_ptr_array_new();
+    
+    while(elements != NULL) {
+        g_ptr_array_add(vector, g_strdup(elements->data));
+        elements = elements->next;
+    }
+    
+    g_ptr_array_add(vector, NULL);
+    return (gchar **)g_ptr_array_free(vector, FALSE);
+}
+
+static void
+gst_playback_handle_missing_elements_failed(GstPlayback *engine)
+{
+    gst_playback_nuke_slist(engine->missing_element_details);
+    engine->missing_element_details = NULL;
+    gst_element_set_state(engine->playbin, GST_STATE_READY);
+    
+    if(engine->error_cb != NULL) {
+       engine->error_cb(engine, GST_CORE_ERROR, GST_CORE_ERROR_MISSING_PLUGIN, NULL, NULL);
+    }
+}
+
+static void
+gst_playback_handle_missing_elements_installer_result(GstInstallPluginsReturn result, gpointer data)
+{
+    GstPlayback *engine = (GstPlayback *)data;
+    
+    g_return_if_fail(IS_GST_PLAYBACK(engine));
+    
+    // TODO: Actually handle a successful plugin installation
+    // if(result == GST_INSTALL_PLUGINS_SUCCESS) {
+    // }
+    
+    engine->install_plugins_noprompt = TRUE;
+    
+    gst_playback_handle_missing_elements_failed(engine);
+    
+    gst_install_plugins_context_free(engine->install_plugins_context);
+    engine->install_plugins_context = NULL;
+}
+
+static void
+gst_playback_handle_missing_elements(GstPlayback *engine)
+{
+    GstInstallPluginsReturn install_return;
+    gchar **details;
+    
+    if(engine->install_plugins_context != NULL) {
+        return;
+    } else if(engine->install_plugins_noprompt) {
+        gst_playback_handle_missing_elements_failed(engine);
+        return;
+    }
+    
+    details = gst_playback_missing_element_details_vectorize(engine->missing_element_details);
+    engine->install_plugins_context = gst_install_plugins_context_new();
+    
+    #ifdef GDK_WINDOWING_X11
+    if(engine->window != NULL) {
+        gst_install_plugins_context_set_xid(engine->install_plugins_context, 
+        GDK_WINDOW_XWINDOW(engine->window));
+    }
+    #endif
+    
+    install_return = gst_install_plugins_async(details, engine->install_plugins_context, 
+        gst_playback_handle_missing_elements_installer_result, engine);
+    
+    if(install_return != GST_INSTALL_PLUGINS_STARTED_OK) {
+        gst_playback_handle_missing_elements_failed(engine);
+        
+        gst_install_plugins_context_free(engine->install_plugins_context);
+        engine->install_plugins_context = NULL;
+    } 
+    
+    g_strfreev(details);
+}
+#endif
+
 static gboolean
 gst_playback_bus_callback(GstBus *bus, GstMessage *message, gpointer data)
 {
@@ -97,17 +217,36 @@ gst_playback_bus_callback(GstBus *bus, GstMessage *message, gpointer data)
             GError *error;
             gchar *debug;
             
+            // FIXME: This is to work around a bug in qtdemux in
+            // -good <= 0.10.6
+            if(message->src != NULL && message->src->name != NULL &&
+                strncmp(message->src->name, "qtdemux", 7) == 0) {
+                break;
+            }
+            
             gst_playback_destroy_pipeline(engine);
             
             if(engine->error_cb != NULL) {
                 gst_message_parse_error(message, &error, &debug);
-                engine->error_cb(engine, error->message, debug);
+                engine->error_cb(engine, error->domain, error->code, error->message, debug);
                 g_error_free(error);
                 g_free(debug);
             }
             
             break;
-        }        
+        } 
+        
+        #ifdef HAVE_GST_PBUTILS
+        case GST_MESSAGE_ELEMENT: {
+            if(gst_is_missing_plugin_message(message)) {
+                engine->missing_element_details = g_slist_append(engine->missing_element_details, 
+                    gst_missing_plugin_message_get_installer_detail(message));
+            }
+            
+            break;
+        }
+        #endif
+        
         case GST_MESSAGE_EOS:
             if(engine->eos_cb != NULL) {
                 engine->eos_cb(engine);
@@ -116,6 +255,15 @@ gst_playback_bus_callback(GstBus *bus, GstMessage *message, gpointer data)
         case GST_MESSAGE_STATE_CHANGED: {
             GstState old, new, pending;
             gst_message_parse_state_changed(message, &old, &new, &pending);
+            
+            #ifdef HAVE_GST_PBUTILS
+            if(old == GST_STATE_READY && new == GST_STATE_PAUSED) {
+                if(engine->missing_element_details != NULL) {
+                    gst_playback_handle_missing_elements(engine);
+                }
+            }
+            #endif
+            
             if(engine->state_changed_cb != NULL) {
                 engine->state_changed_cb(engine, old, new, pending);
             }
@@ -130,6 +278,20 @@ gst_playback_bus_callback(GstBus *bus, GstMessage *message, gpointer data)
                 g_warning("Could not get completion percentage from BUFFERING message");
                 break;
             }
+            
+            if(buffering_progress >= 100) {
+                engine->buffering = FALSE;
+                if(engine->target_state == GST_STATE_PLAYING) {
+                    gst_element_set_state(engine->playbin, GST_STATE_PLAYING);
+                }
+            } else if(!engine->buffering && engine->target_state == GST_STATE_PLAYING) {
+                GstState current_state;
+                gst_element_get_state(engine->playbin, &current_state, NULL, 0);
+                if(current_state == GST_STATE_PLAYING) {
+                    gst_element_set_state(engine->playbin, GST_STATE_PAUSED);
+                }
+                engine->buffering = TRUE;
+            } 
 
             if(engine->buffering_cb != NULL) {
                 engine->buffering_cb(engine, buffering_progress);
@@ -235,6 +397,11 @@ gst_playback_construct(GstPlayback *engine)
 
     audiosink = gst_element_factory_make("gconfaudiosink", "audiosink");
     g_return_val_if_fail(audiosink != NULL, FALSE);
+        
+    /* Set the profile to "music and movies" (gst-plugins-good 0.10.3) */
+    if(g_object_class_find_property(G_OBJECT_GET_CLASS(audiosink), "profile")) {
+        g_object_set(G_OBJECT(audiosink), "profile", 1, NULL);
+    }
     
     engine->audiobin = gst_bin_new("audiobin");
     g_return_val_if_fail(engine->audiobin != NULL, FALSE);
@@ -324,16 +491,6 @@ gst_playback_new()
         return NULL;
     }
     
-    engine->eos_cb = NULL;
-    engine->error_cb = NULL;
-    engine->state_changed_cb = NULL;
-    engine->iterate_cb = NULL;
-    engine->buffering_cb = NULL;
-    engine->tag_found_cb = NULL;
-    
-    engine->iterate_timeout_id = 0;
-    engine->cdda_device = NULL;
-    
     return engine;
 }
 
@@ -343,6 +500,7 @@ gst_playback_free(GstPlayback *engine)
     g_return_if_fail(IS_GST_PLAYBACK(engine));
     
     if(GST_IS_OBJECT(engine->playbin)) {
+        engine->target_state = GST_STATE_NULL;
         gst_element_set_state(engine->playbin, GST_STATE_NULL);
         gst_object_unref(GST_OBJECT(engine->playbin));
     }
@@ -352,6 +510,16 @@ gst_playback_free(GstPlayback *engine)
         engine->cdda_device = NULL;
     }
     
+    gst_playback_nuke_slist(engine->missing_element_details);
+    engine->missing_element_details = NULL;
+    
+    #ifdef HAVE_GST_PBUTILS
+    if(engine->install_plugins_context != NULL) {
+        gst_install_plugins_context_free(engine->install_plugins_context);
+        engine->install_plugins_context = NULL;
+    }
+    #endif
+    
     g_free(engine);
     engine = NULL;
 }
@@ -443,6 +611,7 @@ gst_playback_open(GstPlayback *engine, const gchar *uri)
     
     gst_element_get_state(engine->playbin, &state, NULL, 0);
     if(state >= GST_STATE_PAUSED) {
+        engine->target_state = GST_STATE_READY;
         gst_element_set_state(engine->playbin, GST_STATE_READY);
     }
     
@@ -454,7 +623,10 @@ gst_playback_stop(GstPlayback *engine)
 {
     g_return_if_fail(IS_GST_PLAYBACK(engine));
     gst_playback_stop_iterate_timeout(engine);
-    gst_element_set_state(engine->playbin, GST_STATE_PAUSED);
+    if(GST_IS_ELEMENT(engine->playbin)) {
+        engine->target_state = GST_STATE_PAUSED;
+        gst_element_set_state(engine->playbin, GST_STATE_PAUSED);
+    }
 }
 
 void
@@ -462,6 +634,7 @@ gst_playback_pause(GstPlayback *engine)
 {
     g_return_if_fail(IS_GST_PLAYBACK(engine));
     gst_playback_stop_iterate_timeout(engine);
+    engine->target_state = GST_STATE_PAUSED;
     gst_element_set_state(engine->playbin, GST_STATE_PAUSED);
 }
 
@@ -469,6 +642,7 @@ void
 gst_playback_play(GstPlayback *engine)
 {
     g_return_if_fail(IS_GST_PLAYBACK(engine));
+    engine->target_state = GST_STATE_PLAYING;
     gst_element_set_state(engine->playbin, GST_STATE_PLAYING);
     gst_playback_start_iterate_timeout(engine);
 }
@@ -478,8 +652,8 @@ gst_playback_set_volume(GstPlayback *engine, gint volume)
 {
     gdouble act_volume;
     g_return_if_fail(IS_GST_PLAYBACK(engine));
-	act_volume = CLAMP(volume, 0, 100) / 100.0;
-	g_object_set(G_OBJECT(engine->playbin), "volume", act_volume, NULL);
+    act_volume = CLAMP(volume, 0, 100) / 100.0;
+    g_object_set(G_OBJECT(engine->playbin), "volume", act_volume, NULL);
 }
 
 gint
@@ -570,3 +744,18 @@ gst_playback_get_pipeline_elements(GstPlayback *engine, GstElement **playbin, Gs
     
     return TRUE;
 }
+
+void
+gst_playback_set_gdk_window(GstPlayback *engine, GdkWindow *window)
+{
+    engine->window = window;
+}
+
+void
+gst_playback_get_error_quarks(GQuark *core, GQuark *library, GQuark *resource, GQuark *stream)
+{
+    *core = GST_CORE_ERROR;
+    *library = GST_LIBRARY_ERROR;
+    *resource = GST_RESOURCE_ERROR;
+    *stream = GST_STREAM_ERROR;
+}
