@@ -6,12 +6,13 @@ using System.Collections.Generic;
 using Banshee.Base;
 using Banshee.Sources;
 using Banshee.Database;
+using Banshee.Playlists;
 
 using Mono.Unix;
 
 namespace Banshee.SmartPlaylist
 {
-    public class SmartPlaylistSource : Banshee.Sources.ChildSource
+    public class SmartPlaylistSource : Banshee.Sources.ChildSource, IPlaylist
     {
         private List<TrackInfo> tracks = new List<TrackInfo>();
         private ArrayList watchedPlaylists = new ArrayList();
@@ -104,7 +105,7 @@ namespace Banshee.SmartPlaylist
             get { return tracks; }
         }
 
-        private static Gdk.Pixbuf icon = Gdk.Pixbuf.LoadFromResource("source-smart-playlist.png");
+        private static Gdk.Pixbuf icon = Gdk.Pixbuf.LoadFromResource("source-smart-playlist-22.png");
         public override Gdk.Pixbuf Icon {
             get { return icon; }
         }
