@@ -88,8 +88,19 @@ namespace Banshee.Base
             
             Banshee.Metadata.MetadataService.Instance.Settings = new GlobalMetadataSettings();
             
+            // move the legacy app data directory (~/.gnome2/banshee) to the new 
+            // one (~/.config/banshee) or just create the new one
             if(!Directory.Exists(Paths.ApplicationData)) {
-                Directory.CreateDirectory(Paths.ApplicationData);
+                if(Directory.Exists(Paths.LegacyApplicationData)) {
+                    Directory.Move(Paths.LegacyApplicationData, Paths.ApplicationData);
+                    LogCore.Instance.PushInformation("Migrated Settings", 
+                        String.Format("Moved the Banshee's data directory from `{0}' to `{1}'",
+                        Paths.LegacyApplicationData, Paths.ApplicationData), false);
+                }
+            
+                if(!Directory.Exists(Paths.ApplicationData)) {
+                    Directory.CreateDirectory(Paths.ApplicationData);
+                }
             }
             
             // override the browser URI launch hook in Last.FM and Banshee.Widgets.LinkLabel
@@ -164,10 +175,12 @@ namespace Banshee.Base
             startup.Register(Catalog.GetString("Initializing CD writing support"), true, 
                 Catalog.GetString("CD burning support will be disabled for this instance"), Banshee.Burner.BurnerCore.Initialize);
                 
+            startup.Register(Catalog.GetString("Starting background tasks"), PowerManagement.Initialize);
+            startup.Register(Catalog.GetString("Starting background tasks"), GnomeMMKeys.Initialize);
+                
             startup.Register(Catalog.GetString("Initializing plugins"), Banshee.SmartPlaylist.SmartPlaylistCore.Instance.Initialize);
             startup.Register(Catalog.GetString("Initializing plugins"), Banshee.Plugins.PluginCore.Initialize);
             startup.Register(Catalog.GetString("Initializing scripts"), Banshee.Plugins.ScriptCore.Initialize);
-            startup.Register(Catalog.GetString("Starting background tasks"), PowerManagement.Initialize);
             
             if(interfaceStartupHandler != null) {
                 startup.Register(Catalog.GetString("Loading user interface"), interfaceStartupHandler);
@@ -323,6 +336,11 @@ namespace Banshee.Base
             string env_val = Environment.GetEnvironmentVariable(env);
             return env_val != null && env_val != String.Empty;
         }
+        
+        private static System.Globalization.CultureInfo culture_info = new System.Globalization.CultureInfo("en-US");
+        public static System.Globalization.CultureInfo InternalCultureInfo {
+            get { return culture_info; }
+        }
     }
     
     public static class InterfaceElements
