@@ -27,8 +27,10 @@
  */
  
 using System;
+using System.Text.RegularExpressions;
 using Mono.Unix;
 
+using Banshee.Metadata;
 using Banshee.Configuration.Schema;
 
 namespace Banshee.Base
@@ -46,19 +48,8 @@ namespace Banshee.Base
         }
     }
         
-    public class UnsupportedMimeTypeException : ApplicationException
-    {
-        public UnsupportedMimeTypeException(string msg) : base(msg)
-        {
-        }
-    }
-    
     public static class StreamTagger
     {
-        private static string [] valid_mimetype_prefixes = new string [] {
-            "audio/", "application/", "taglib/"
-        };
-    
         public static TagLib.File ProcessUri(SafeUri uri)
         {
             string mimetype = null;
@@ -69,22 +60,11 @@ namespace Banshee.Base
             } catch {
             }
 
-            if(mimetype != null) {
-                process = false;
-                foreach(string prefix in valid_mimetype_prefixes) {
-                    if(mimetype.StartsWith(prefix)) {
-                        process = true;
-                        break;
-                    }
-                }
-                
-                if(!process) {
-                    throw new UnsupportedMimeTypeException(mimetype);
-                }
-            }
+            TagLib.File file = Banshee.IO.IOProxy.OpenFile(uri.IsLocalPath ? uri.LocalPath : uri.AbsoluteUri, mimetype, TagLib.ReadStyle.Average);
 
-            return TagLib.File.Create(uri.IsLocalPath ? uri.LocalPath : uri.AbsoluteUri, 
-                mimetype, TagLib.AudioProperties.ReadStyle.Average);
+            if (file.Properties.MediaTypes != TagLib.MediaTypes.Audio)
+                throw new TagLib.UnsupportedFormatException ("File doesn't contain only audio");
+            return file;
         }
     
         private static string Choose(string priority, string fallback)
@@ -94,7 +74,7 @@ namespace Banshee.Base
         
         public static void TrackInfoMerge(TrackInfo track, TagLib.File file)
         {
-            track.Artist = Choose(file.Tag.JoinedArtists, track.Artist);
+            track.Artist = Choose(file.Tag.JoinedPerformers, track.Artist);
             track.Album = Choose(file.Tag.Album, track.Album);
             track.Title = Choose(file.Tag.Title, track.Title);
             track.Genre = Choose(file.Tag.FirstGenre, track.Genre);
@@ -103,11 +83,9 @@ namespace Banshee.Base
             track.Year = (int)file.Tag.Year;
             track.MimeType = file.MimeType;
             
-            if(file.AudioProperties != null) {
-                track.Duration = file.AudioProperties.Duration;
-            }
+            track.Duration = file.Properties.Duration;
         }
-    
+   
         public static void TrackInfoMerge(TrackInfo track, StreamTag tag)
         {
             try {
@@ -115,8 +93,30 @@ namespace Banshee.Base
                     case CommonTags.Artist:
                         track.Artist = Choose((string)tag.Value, track.Artist);
                         break;
-                    case CommonTags.Title:
-                        track.Title = Choose((string)tag.Value, track.Title);
+                    case CommonTags.Title:                        
+                        string title = Choose((string)tag.Value, track.Title);
+
+                        // Try our best to figure out common patterns in poor radio metadata.
+                        // Often only one tag is sent on track changes inside the stream, 
+                        // which is title, and usually contains artist and track title, separated
+                        // with a " - " string.
+                        if(track.IsLive && title.Contains(" - ")) {
+                            string [] parts = Regex.Split(title, " - ");
+                            track.Title = parts[1].Trim();
+                            track.Artist = parts[0].Trim();
+                            
+                            // Often, the title portion contains a postfix with more information
+                            // that will confuse lookups, such as "Talk [Studio Version]".
+                            // Strip out the [] part.
+                            Match match = Regex.Match(track.Title, "^(.+)[ ]+\\[.*\\]$");
+                            if(match.Groups.Count == 2) {
+                                track.Title = match.Groups[1].Value;
+                            }                            
+                            
+                            MetadataService.Instance.Lookup(track);
+                        } else {
+                            track.Title = title;
+                        }
                         break;
                     case CommonTags.Album:
                         track.Album = Choose((string)tag.Value, track.Album);
@@ -184,7 +184,7 @@ namespace Banshee.Base
             }
         
             TagLib.File file = StreamTagger.ProcessUri(track.Uri);
-            file.Tag.AlbumArtists = new string [] { track.Artist };
+            file.Tag.Performers = new string [] { track.Artist };
             file.Tag.Album = track.Album;
             file.Tag.Genres = new string [] { track.Genre };
             file.Tag.Title = track.Title;
